#
# TV::Search - find programmes matching certain criteria
# ~~~~~~~~~~   (c) Andrew Flegg 2002. Released under the Artistic Licence.
#
# where <flags> is defined, format is:
#    bit       meaning
#     0        title
#     1        subtitle
#     2        description
#     3        type

package TV::Search;

use TV::Core;
use TV::Format;


# -- We're created --------------------------------------------------------
#
sub new {
    my $class   = shift;
    my (%parm)  = @_;
    my $self    = bless \%parm, $class;

    my $days     = $self->{days} or die "$class: no days specified\n";
    my $channels = $self->{channels} or die "$class: no channels specified\n";

    return $self;
}


# -- Fill in the array ----------------------------------------------------
#
sub _readData {
    my $self     = shift;
    my $re       = shift;
    my $days     = $self->{days};
    my $channels = $self->{channels};

    # -- Use fgrep to speed it all up ----------------------------
    #
    my $paths    = '';
    my $files    = '';
    foreach (sort(@$days)) {
        s/[^\d-]+//g;
        $paths .= " '".$TV::Core::DATA_DIR."/listings/$_'";
    }

    foreach (@$channels) {
        s/[^A-Za-z0-9\.\_-]+//g;
        $files .= ' -o' if $files;
        $files .= " -name '$_.xml'";
    }
   
    my ($FIND, $XARGS, $EGREP) = qw(/usr/bin/find /usr/bin/xargs /bin/egrep);
    open(GREP, "$FIND $paths -type f $files | $XARGS -i $EGREP -l -i \Q$re\E {} |") or die "Unable to open pipe: $!\n";
    while (<GREP>) {
        next unless m!\b(-?\d+)/([^/]+)\.xml$!;
        next if $self->{data}->{$1}->{$2};
        $self->{data}->{$1}->{$2} = TV::Format->new(channel => $2,
                                                    day     => $1);
    }

    return;
}


# -- Find programmes which match a regular expression ---------------------
#
sub findRegExp {
    my $self  = shift;
    my $re    = shift;
    my $flags = shift;

    $self->_readData($re);

    my @list     = ();
    my $days     = $self->{days} || [];
    my $channels = $self->{channels} || [];
    my %descs    = ();

    $| = 1;
    my $pat      = qr/$re/i;
    foreach my $d (sort(@$days)) {
        foreach my $c (@$channels) {
            my $obj = $self->{data}->{$d}->{$c} or next;
            foreach my $p (@{ $obj->{listings} }) {
                my $add = 0;
                $add++ if ($flags & 1) and ($p->{title} || '') =~ /$pat/;
                $add++ if ($flags & 2) and ($p->{subtitle} || '') =~ /$pat/;
                $add++ if ($flags & 4) and ($p->{desc} || '') =~ /$pat/;
                $add++ if ($flags & 8) and ($p->{type} || '') =~ /$pat/;

                next unless $add;
		$p->{abstime} = ($p->{day} * 10000) + $p->{start};
                my $got = ($p->{desc} ? $descs{$p->{desc}} : 0);
                if ($got && ($got->{title} eq $p->{title})) {
                    push @{ $got->{repeats} }, $p;
                } else {                    
                    push @list, $p;
                    $descs{$p->{desc}} = $p if $p->{desc};
                }
            }
        }
    }

    @list = sort { $a->{abstime} <=> $b->{abstime} } @list;

    return \@list;
}



1;
